#gba.ps1 v5.5

#Default Character to Determine Unique ROM Name (Blank = Skip)
$romSplitchar = "("

#Unzip ROMs That Do Not Support .ZIP Files On RetroPie
$romUnzip = $false

#Any Exact ROM Name (Including Extension) In This List Will Not Be Deleted
#-ROMs listed here will NOT be deleted even if they match a value in romsExceptions
#-ROMs listed here will NOT be deleted if it's NOT the Primary ROM via romsSortOrder
#-Ex: "Castlevania (USA) (Rev A).zip"
$romsDoNotDelete = @(
	"4 Games on One Game Pak (Nicktoons) (USA).zip"
	,"4 Games on One Game Pak (Racing) (USA) (En,Fr,De,Es,It).zip"
)

#Any ROMs Containing These Strings Or Exact ROM Name Will Be Removed Unless the Exact ROM Name Is In romsDoNotDelete
$romsExceptions = @(
	"[BIOS]"
	,"[b]"
    ,"(Alpha)"
	,"(Beta)"
	,"(Beta 1)"
	,"(Beta 2)"
	,"(Beta 3)"
	,"(Proto)"
	,"(Proto 1)"
	,"(Proto 2)"
	,"(Promo)"
	,"(Sample)"
	,"(Debug Version)"
	,"(Enhancement Chip)"
	,"(GameCube Edition)"
	,"(Promo, Virtual Console)"
	,"(Virtual Console)"
	,"(Wii Virtual Console)"
	,"(Wii U Virtual Console)"
	,"(Asia)"
	,"(Australia)"
	,"(Brazil)"
	,"(Canada)"
	,"(China)"
	,"(France)"
	,"(Germany)"
	,"(Hong Kong)"
	,"(Italy)"
	,"(Japan)"
	,"(Korea)"
	,"(Netherlands)"
	,"(Russia)"
	,"(Spain)"
	,"(Sweden)"
	,"(Taiwan)"
	,"Action Replay" #Not a Game (RAM Modification Cheat Cartridge)
	,"Game Boy Advance Video" #Not a Game (Movie/TV Shows)
	#Demo/Test/Misc Cartridge
	,"Aging Cartridge (World).zip"
	,"Basic Sample by Nintendo (Unknown) (Demo).zip"
	,"Dolphin Demo by Nintendo (Unknown) (Demo).zip"
	,"e-Reader (USA).zip"
	,"Mario Kart - Double Dash!! (USA) (Fire Emblem GBA - Bonus Items).zip" #Bonus Disk
	,"Nintendo MP3 Player (Europe) (En,Fr,De,Es,It).zip"
	,"Nintendo Puzzle Collection Controller (USA) (GameCube).zip"
	#RAM Modification Cheat Cartridge
	,"CodeBreaker (USA) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"GameShark GBA (USA) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"GameShark GBA (USA) (Alt 1) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"GBA Personal Organizer (USA) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Nintendogs + Super Mario 64 DS - Trainer Expansion for NDS (USA).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Xploder Advance (Europe) (Alt 1) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Xploder Advance (Europe) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	#Duplicate ROMSs with Alternate Names
	,"2 Game Pack! - Uno & Skip-Bo (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for '2 Game Pack! - Uno + Skip-Bo (USA).zip'
	,"2 Games in 1 - Dr. Mario + Puzzle League (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for '2 Games in One! - Dr. Mario + Puzzle League (USA, Australia).zip'
	,"2 in 1 Game Pack - Spider-Man & Spider-Man 2 (Europe) (En,Fr,De+En,Fr,De,Es,It).zip" #Alternate Name for '2 in 1 Game Pack - Spider-Man + Spider-Man 2 (USA) (En,Fr,De+En,Fr,De,Es).zip'
	,"Crash & Spyro Superpack - Spyro Orange - The Cortex Conspiracy + Crash Bandicoot Purple - Ripto's Rampage (USA).zip" #Alternate Name for 'Crash & Spyro Super Pack Volume 3 (Europe) (En,Fr,De,Es,It).zip'
	,"Crash Bandicoot Fusion (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Crash Bandicoot Purple - Ripto's Rampage (USA).zip'
	,"Defender - For All Mankind (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Defender (USA).zip'
	,"ESPN Winter X-Games Snowboarding 2 (Europe).zip" #Alternate Name for 'ESPN Winter X-Games Snowboarding 2002 (USA).zip'
	,"Eggo Mania (Europe) (En,Fr,De,Es,It,Nl).zip" #Alternate Name for 'Egg Mania (USA) (En,Fr,Es).zip'
	,"Freestyle Scooter (Europe).zip" #Alternate Name for 'Razor Freestyle Scooter (USA).zip'
	,"GT Championship (Europe).zip" #Alternate Name for 'Top Gear GT Championship (USA).zip'
	,"Ice Climber (USA) (Advance Play Edition).zip" #Alternate Name for 'Classic NES Series - Ice Climber (USA, Europe).zip'
	,"Jungle Book 2, The (Europe) (En,Fr,De,Es,It,Nl).zip" #Alternate Name for 'Jungle Book, The (USA) (En,Fr,De,Es,It,Nl).zip'
	,"Jurassic Park III - Dino Attack (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Jurassic Park III - Island Attack (USA).zip'
	,"Kim Possible (Europe) (En,Fr,De,Es) (Rev 1).zip" #Alternate Name for 'Kim Possible - Revenge of Monkey Fist (USA).zip'
	,"Kim Possible (Europe) (En,Fr,De,Es).zip" #Alternate Name for 'Kim Possible - Revenge of Monkey Fist (USA).zip'
	,"Kim Possible (Japan).zip" #Alternate Name for 'Kim Possible - Revenge of Monkey Fist (USA).zip'
	,"Konami Collector's Series - Arcade Classics (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Konami Collector's Series - Arcade Advanced (USA).zip'
	,"Lilo & Stitch 2 (Europe) (En,Fr,De,Es).zip" #Alternate Name for 'Lilo & Stitch 2 - Haemsterviel Havoc (USA).zip'
	,"Lizzie McGuire (Europe) (En,Fr,De,Es).zip" #Alternate Name for 'Lizzie McGuire - On the Go! (USA).zip'
	,"Maniac Racers Advance (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Motocross Maniacs Advance (USA) (En,Es).zip'
	,"Mario Power Tennis (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Mario Tennis - Power Tour (USA, Australia) (En,Fr,De,Es,It).zip'
	,"Mega Man Battle Network 3 - Blue (Europe).zip" #Alternate Name for 'Mega Man Battle Network 3 - Blue Version (USA).zip'
	,"Mega Man Battle Network 3 - White (Europe).zip" #Alternate Name for 'Mega Man Battle Network 3 - White Version (USA).zip'
	,"Pokemon - Liechi Berry Glitch Fix & Shiny Zigzagoon Distribution (Europe).zip" #Alternate Name for 'Pokemon - LeafGreen Version (USA).zip'
	,"Pro Beach Soccer (Europe) (En,Fr,De,Es,It,Pt).zip" #Alternate Name for 'Ultimate Beach Soccer (USA).zip'
	,"Pro Tennis WTA Tour (Europe).zip" #Alternate Name for 'WTA Tour Tennis (USA).zip'
	,"Rayman - Hoodlums' Revenge (Europe) (En,Fr,De,Es,It,Nl).zip" #Alternate Name for 'Rayman - Hoodlum's Revenge (USA) (En,Fr,Es).zip'
	,"Scrabble Scramble! (Europe) (En,Fr,De,Es,It,Nl).zip" #Alternate Name for 'Scrabble Blast! (USA).zip'
	,"Scrabble Scramble! (Europe).zip" #Alternate Name for 'Scrabble Blast! (USA).zip'
	,"SpongeBob SquarePants and Friends - Battle for Volcano Island (Europe) (En,Fr,De,Es,It,Nl).zip" #Alternate Name for 'Nicktoons - Battle for Volcano Island (USA).zip'
	,"SpongeBob SquarePants and Friends Unite! (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Nicktoons Unite! (USA).zip'
	,"SpongeBob SquarePants and Friends in Freeze Frame Frenzy (Europe) (En,Fr,De,Es,Nl).zip" #Alternate Name for 'Nicktoons - Freeze Frame Frenzy (USA).zip'
	,"SpongeBob and Friends - Attack of the Toybots (Europe) (En,De).zip" #Alternate Name for 'Nicktoons - Attack of the Toybots (USA).zip'
	,"Spyro Fusion (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Spyro Orange - The Cortex Conspiracy (USA).zip'
	,"Super Black Bass Advance (Europe).zip" #Alternate Name for 'American Bass Challenge (USA).zip'
	,"Super Mario Bros. (USA) (Advance Play Edition).zip" #Alternate Name for 'Classic NES Series - Super Mario Bros. (USA, Europe).zip'
	,"Super Mario Ball (Europe).zip" #Alternate Name for 'Mario Pinball Land (USA, Australia).zip'
	,"TG Rally (Europe).zip" #Alternate Name for 'Top Gear Rally (USA).zip'
	,"TMNT - Teenage Mutant Ninja Turtles (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'TMNT (USA) (En,Fr,Es).zip'
	,"Tiny Toon Adventures - Buster's Bad Dream (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Tiny Toon Adventures - Scary Dreams (USA).zip'
	,"Yu-Gi-Oh! - Ultimate Masters Edition - World Championship Tournament 2006 (Europe) (En,Ja,Fr,De,Es,It).zip" #Alternate Name for 'Yu-Gi-Oh! - Ultimate Masters - World Championship Tournament 2006 (USA) (En,Ja,Fr,De,Es,It).zip'
	,"Zapper (Europe) (En,Fr,De,Es,It).zip" #Alternate Name for 'Zapper - One Wicked Cricket! (USA).zip'
)

#Preferred Sorting Order to Pick 1 Primary ROM to Keep When Evaluating Duplicates
#-Order Selects Specific Names followed by more generalized names based around language
#-ROMs in romsExceptions will not be deleted even if it is not determined as the primary
#-Ex: "Joust (USA).zip" would be primary over "Joust (Japan).zip"
$romsSortOrder = @(
	#---------------------------------------
	#---USA PREFERRED ORDER---
	#---------------------------------------
	#(USA).zip
	"\(USA\)\.zip"
	#(USA) (Unl).zip
	,"\(USA\)\s*\(Unl\)\.zip"
	#(USA) (En,*).zip
	,"\(USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA) (En,*) (Unl).zip
	,"\(USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA, Europe).zip
	,"\(USA,\s*Europe\)\.zip"
	#(USA, Europe) (Unl).zip
	,"\(USA,\s*Europe\)\s*\(Unl\)\.zip"
	#(USA, Europe) (En,*).zip
	,"\(USA,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Europe) (En,*) (Unl).zip
	,"\(USA,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Europe) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Europe\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan, USA).zip
	,"\(Japan,\s*USA\)\.zip"
	#(Japan, USA) (Unl).zip
	,"\(Japan,\s*USA\)\s*\(Unl\)\.zip"
	#(Japan, USA) (En,*).zip
	,"\(Japan,\s*USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan, USA) (En,*) (Unl).zip
	,"\(Japan,\s*USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan, USA) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan,\s*USA\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA, Asia).zip
	,"\(USA,\s*Asia\)\.zip"
	#(USA, Asia) (Unl).zip
	,"\(USA,\s*Asia\)\s*\(Unl\)\.zip"
	#(USA, Asia) (En,*).zip
	,"\(USA,\s*Asia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Asia) (En,*) (Unl).zip
	,"\(USA,\s*Asia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Asia) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Asia\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"	
	#(USA, Australia).zip
	,"\(USA,\s*Australia\)\.zip"
	#(USA, Australia) (Unl).zip
	,"\(USA,\s*Australia\)\s*\(Unl\)\.zip"
	#(USA, Australia) (En,*).zip
	,"\(USA,\s*Australia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Australia) (En,*) (Unl).zip
	,"\(USA,\s*Australia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Australia) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Australia\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA, Korea).zip
	,"\(USA,\s*Korea\)\.zip"
	#(USA, Korea) (Unl).zip
	,"\(USA,\s*Korea\)\s*\(Unl\)\.zip"
	#(USA, Korea) (En,*).zip
	,"\(USA,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Korea) (En,*) (Unl).zip
	,"\(USA,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Korea) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Korea\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA) (Rev #)
	,"^(?=.*?\(USA\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(USA) (Rev A)
	,"^(?=.*?\(USA\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(USA) (Rev #A)
	,"^(?=.*?\(USA\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(USA) (Alt) OR (Alternate)
	,"^(?=.*?\(USA\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(USA) (Alt #)
	,"^(?=.*?\(USA\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(USA)
	,"\(USA\)"
	#(USA, Europe)
	,"\(USA,\s*Europe\)"
	#(Japan, USA)
	,"\(Japan,\s*USA\)"
	#(USA, Asia)
	,"\(USA,\s*Asia\)"
	#(USA, Australia)
	,"\(USA,\s*Australia\)"
	#(USA, Korea)
	,"\(USA,\s*Korea\)"	
	#(*USA*)
	,"\((.*?)USA(.*?)\)"
	#---------------------------------------
	#---WORLD PREFERRED ORDER---
	#---------------------------------------
	#(World).zip
	"\(World\)\.zip"
	#(World) (Unl).zip
	,"\(World\)\s*\(Unl\)\.zip"
	#(World) (En,*).zip
	,"\(World\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(World) (En,*) (Unl).zip
	,"\(World\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(World) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(World\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(World) (Rev #)
	,"^(?=.*?\(World\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(World) (Rev A)
	,"^(?=.*?\(World\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(World) (Rev #A)
	,"^(?=.*?\(World\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(World) (Alt) OR (Alternate)
	,"^(?=.*?\(World\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(World) (Alt #)
	,"^(?=.*?\(World\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(World)
	,"\(World\)"
	#(*World*)
	,"\((.*?)World(.*?)\)"
	#---------------------------------------
	#---EUROPE PREFERRED ORDER---
	#---------------------------------------
	#(Europe).zip
	"\(Europe\)\.zip"
	#(Europe) (Unl).zip
	,"\(Europe\)\s*\(Unl\)\.zip"
	#(Europe) (En,*).zip
	,"\(Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Europe) (En,*) (Unl).zip
	,"\(Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Europe) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Europe\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan, Europe).zip
	,"\(Japan,\s*Europe\)\.zip"
	#(Japan, Europe) (Unl).zip
	,"\(Japan,\s*Europe\)\s*\(Unl\)\.zip"
	#(Japan, Europe) (En,*).zip
	,"\(Japan,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan, Europe) (En,*) (Unl).zip
	,"\(Japan,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan, Europe) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan,\s*Europe\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Europe) (Rev #)
	,"^(?=.*?\(Europe\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(Europe) (Rev A)
	,"^(?=.*?\(Europe\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(Europe) (Rev #A)
	,"^(?=.*?\(Europe\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(Europe) (Alt) OR (Alternate)
	,"^(?=.*?\(Europe\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(Europe) (Alt #)
	,"^(?=.*?\(Europe\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(Europe)
	,"\(Europe\)"
	#(*Europe*)
	,"\((.*?)Europe(.*?)\)"
	#---------------------------------------
	#---JAPAN PREFERRED ORDER---
	#---------------------------------------
	#(Japan).zip
	"\(Japan\)\.zip"
	#(Japan) (Unl).zip
	,"\(Japan\)\s*\(Unl\)\.zip"
	#(Japan) (En,*).zip
	,"\(Japan\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan) (En,*) (Unl).zip
	,"\(Japan\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan, Korea).zip
	,"\(Japan,\s*Korea\)\.zip"
	#(Japan, Korea) (Unl).zip
	,"\(Japan,\s*Korea\)\s*\(Unl\)\.zip"
	#(Japan, Korea) (En,*).zip
	,"\(Japan,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan, Korea) (En,*) (Unl).zip
	,"\(Japan,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan, Korea) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan,\s*Korea\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan) (Rev #)
	,"^(?=.*?\(Japan\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(Japan) (Rev A)
	,"^(?=.*?\(Japan\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(Japan) (Rev #A)
	,"^(?=.*?\(Japan\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(Japan) (Alt) OR (Alternate)
	,"^(?=.*?\(Japan\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(Japan) (Alt #)
	,"^(?=.*?\(Japan\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(Japan)
	,"\(Japan\)"
	#(*Japan*)
	,"\((.*?)Japan(.*?)\)"
	#---------------------------------------
	#---MISC PREFERRED ORDER---
	#---------------------------------------
	#(Unknown)
	,"\(Unknown\)"
	#---------------------------------------
	#---LAST RESORT VALID ROM EXTENSION PREFERRED ORDER---
	#---------------------------------------
	,"\.zip"
	,"\.7z"
	,"\.gba"
)
